# -*- coding: utf-8 -*-
"""
Created on Thu Nov 16 16:49:19 2017

@author: gerd doeben-henisch

Corresponding article: AAI - Case Study. Actor Story with Actor Model. Simple Grid-Environment. eJournal SW4IEF- ISSN 2567-6458. Nov-15, 2017. URL: www.uffmm.org

STRUCTURE OF PROGRAM

COLLECTING PARAMETERS

- Which type of actor ('name')
- Which type of environment (random/ 'name')
-- If random: Size of Grid
-- If random: Percentage of obstacles
-- If random: Percentage of food
-- Compute environment for to  enable clicking for startposition
-- If random: Startposition
- Number of experiments (NEXP)

EXPERIMENT

- Actor is moving around until he reaches the end of the defined cycle-time or he died on account of lack of energy
- NEXP-many repetitions

STATISTICS

- Presentation of amount of energy per experiment against Percentage of Obstacles and Food
- Mean value of Energy

COORDINATES

There are three different areas with coordinates, which interact:

- The PHYSICAL SCREEN starts from (xc,yc) = (0,0) upper left to (distance * GridSizeN,distance * GridSizeN). This physical screen is divided into cells (little rectangles) with length distance. The grid starts upper left with (x,y) = (0,0) and the down right corner of the cell is (0+distance-1, 0+distance-1)
- The GRID-DATA-STRUCTURE as an array of size GridSizeN x GridSizeN. The upper left cell has the coordinates (0,0). This grid-cell (0,0) corresponds to the  physical value (0,0) - (0+distance-1, 0+distance-1)
- The LOGICAL-VISUAL-SCREEN with the upper left cell (x,y) = (1,1)


"""
# ae6-2-3-main.py
#Print-Identifier
PID="MAIN:\n "

def main():
        
    import graphics as grph        #Graphics objects from Zelle
    import environment as env      #Functions to generate grids
    import numpy as np             #numerical library numpy
    import acctor as acc           #Acctor related functions
    import time as tm              #Timer library
    import control as ctrl         #Control windows library
    import pyttsx3 as tt3           #Text-2-Speech module
    import matplotlib.pyplot as plt #PLotlibrary
   
######################################################################
## PREPARING THE FRAMEWORK
#######################################################################
   
    #Start Speech Engine pyttsx3
    #Speaking text
    #call engine
    engine = tt3.init('espeak')
    
    #select voice
    #engine.setProperty('voice', 'german')
    engine.setProperty('voice', 'english-us')
    LNGFLAG = 'en'
 
    ##Generate window with control parameters
    winC = ctrl.ctrlWinStart()

    ## Which type of actor ('name')
    ## Ellowed is here 'fixed rule Right-Handed' = 1
    ## Random behavior = 2

    behaviorNo, txt8,txt8b = ctrl.ctrlWinSelBehave(winC)

    #Which type of environment ('ran' or 'name')
    #The default is actually 'random'

    ## if random: Size of Grid
    ## User-Window: Upper Left (1,1) -> (x,y)
    GridSizeN, txtGSize, txtGSizeb = ctrl.ctrlWinGridSize(winC)
    ## Because on the visibility there is a limit to the grid-size with about 50
    if GridSizeN > 50:
        GridSizeN = 50
    elif GridSizeN < 2:
        GridSizeN = 2
    dimensionX = GridSizeN  
    dimensionY = dimensionX
   
    ##If random: Percentage of obstacles
    nobj, txtPercObj = ctrl.ctrlWinSelObjNo(winC)
   
    #If random: Percentage of food
    nfood, txtPercFood = ctrl.ctrlWinPercFood(winC)
   
    ## Number of Cycles for one experiment
    cyclesNo, txtCylesNo = ctrl.ctrlWinSelCylenumber(winC)
   
    ## Number of Repetitions of experiment
    repeatNo, txtExpRepet = ctrl.ctrlWinSelRepetitions(winC)
   
    ## Select Debug (Single Step Mode)
    debugYes = ctrl.ctrlWinSelSleep(winC)
    if debugYes != 1:
        SLEEP = ctrl.ctrlWinSelVelocity(winC)
    else:
        SLEEP = 1  #Dummy value
   
    ## Do you want Auto-Start (random start position)?
    autoStart = ctrl.ctrlWinSelAutoStart(winC)
   
    #close control-window
    winC.close()
    
    # Open New Control Window for Monitoring
    winC2,txtEnergy,txtTimeLeft  = ctrl.ctrlWinRun()
   
    ###################################################################
    ## PREPARE STATISTICS
    ## (1) Array for Energy values
    ## (2) Mean Energy, Deathrate, 
    #######################################################################

    ENERGYAR = np.zeros((repeatNo,),int)
    OBST = nobj
    FOOD = nfood
   
    #####################################################################
    ## PREPARE ENVIRONMENT
    ##
    ## A graphics window
    ## Upper Left (0,0) -> (xmax, ymax)
    distance = 100   #distance of two lines in the grid of the screen
    MAXSCREEN = 1000
   
    ## If the size of the grid is increasing one has to minimize the cell-size. 
    ## But this can not be done ad infinitum. Thus there will be some LIMIT to 
    ## the size of the grid saying 100 - 20 = 80!
    if GridSizeN > 8:
        distance = int(MAXSCREEN/GridSizeN)
           
    xmax = distance * dimensionX
    ymax = distance * dimensionY
   
    win = grph.GraphWin('ENVIRONMENT', xmax, ymax)
    win.setBackground('HoneyDew')
   
    first = 1    #First element of the list
    last = dimensionX     # Last element of the list used
    ## Data Array Grid
    ## Upper Left (0,0)
    maxXcoord = dimensionX  #size of the data array in the background
    maxYcoord = dimensionY
   
    #generating the lines of the grid
    env.grid(win,xmax, ymax, first, last, distance)
   
    #Fill the grid with spaces and objects
    gr2=env.fillgridobj2(win,distance,first, dimensionX, maxXcoord,maxYcoord,nobj)
   
    # Fill the grid with food
    gr2=env.fillgridfood(win,distance,first, last, maxXcoord,maxYcoord,nfood,gr2)
   
    # Set up a parallel data-environment for the window
   
    envname = 'env1'
    envclock = 0
    foodEnergy = 125  #Amount of energy by intaking 1x food
   
    e1=env.ENVOBJ(envname,envclock, nobj,nfood,foodEnergy)
   



############################################################################ 
## STARTING A LOOP
########################################################################### 
## M-MANY REPETITIONS
## repeatNo
###########################################################################
    FLAGREP = 1
    while FLAGREP < repeatNo+1:
        print(PID+'Number of repetition  ',FLAGREP)

###########################################################################
## ONE EXPERIMENT WITH 
## cyclesNo
###########################################################################
        FLAGCYC = 1
        # Starting the global environment time
        e1.setClock(FLAGCYC)
       
        # Getting actual time
        time=e1.getClock()  
   
        #Introducing a single actor
       
        a1 = acc.introduceActorAuto(engine,LNGFLAG,win,distance,gr2,time,first,last,autoStart,SLEEP,GridSizeN)
        EMAX = a1.getEnergy()
       
        if SLEEP > 0 and autoStart < 1:
            win.getMouse()
       
        while FLAGCYC < cyclesNo+1:
            print(PID+'Number of cycles  ',FLAGCYC)
           
            # Look to the actors  for new responses
    
            # The environment  gets the informtion about the wanted direction
            # from the actors  
            # The actors will compute it's response and will update the class
            # Depending from the behavior type the new direction will be computed differently       
            # Actually there is only ONE actor and this one actor can only
            # compute the next DIRECTION, which is stored in its data-structure  
            # There are two information available for the environment:
            # (1) Actor will move
            # (2) Actor has a direction
    
            ### Actor computes its messages: dir and eill act
           
            if behaviorNo == 1:
                acc.behaviorActFix(e1,a1)
            else:
                acc.behaviorAcRand(e1,a1)
                
            ## Environment asks for this information
                
            if a1.getOutputMessage() == 'yesMove':
                
            ## If this actor wants to move, then the environment has
            ## to compute
            ## (1) whether this is possible (on account of obstacles...)
            ## (2) and when YES, move the actor to the new place
                
                # If there is a response then compute the next possible move
                dirnew = a1.getDir()
                print(PID+'New Direction planned = ',dirnew)
                
                #Check whether this new position  is possible
                # by computing new coordinates
        
                p=a1.getPos()
                print(PID+'Actual Position = ',p)
                xold=p[0] 
                yold=p[1]
                
                ## While actor is still at old position, 
                ##but looking to a new direction
                ## This new looking shall be shown by
                ## by replacing the old circle with a new one
                ## putting the HEAD in the new direction
                
                env.deleteActor1(a1,win,distance,gr2)
                env.introduceActor2Eye1(a1,distance,win,dirnew)
                
                #Compute the next possible move
                #The direction comes from the preceding behavior
    
                Err,xnew,ynew = env.newPosition(a1,dimensionX)
                
                if Err == -1:
                    print(PID+'Err = ', Err)
                else:
                    print(PID+'New Position planned = ', (xnew,ynew))
                    
                ## Analyse the environment
                    
                if gr2[ynew-1,xnew-1] != 0:
                    #Telling the actor that the planned new position is occupied
                    a1.setInputMessage("NoMove")
                    print(PID+'actor has hit an obstacle at ',xnew,ynew)
                    #REMARK: The actor is hitting food only if the food is in the
                    #direction of his move!
                    
                    #talk
                    if debugYes == 1:
                        if LNGFLAG == 'de':
                            engine.say('Hallo User, ')
                            engine.say('Ich stehe vor einem Hindernis.')
                            engine.runAndWait()
                        else:
                            engine.say('Hallo User, ')
                            engine.say('I am standing before an obstacle')
                            engine.runAndWait()
                        
                        
                    if gr2[ynew-1,xnew-1] == 2:
                            print(PID+'actor  has hit FOOD')
                            a1.setIntake(foodEnergy)
                
                            if debugYes == 1:
                                #talk
                                if LNGFLAG == 'de':
                                    engine.say('Hallo User, ')
                                    engine.say('Da ist Futter.')
                                    engine.runAndWait()
                                
                                else:
                                    engine.say('Hallo User, ')
                                    engine.say('This is food')
                                    engine.runAndWait()
                      
                else:
                    TYPEID=3 #actor
                    
                    #First change position in data array
                    # But keep the old position in the actor !
                    print(PID+'actor  will move in Array',xnew,ynew)
                    env.moveArray(TYPEID,a1,xnew,ynew,gr2)

                    #Next change position in window
                    print(PID+'Inserting actor at', xnew,ynew)
                    # First delete the  actor at old position
                    env.deleteActor2(a1,win,distance,gr2)
                 
                    # Then place the actor at the new position
                    env.introduceActor2Eye(a1, xnew,ynew,distance,win,dirnew)
     
                
            ## No move is planned               
            else:
                 print(PID+'No move wanted by actor.')              
           
       
       ####################################################
       ## END OF CYCLE
       ## CHECK PYRAMETERS
       ## eNERGY AND NUMBER OF CYCLE AND TIME
       ###################################################
                                   
            envclock= envclock+1
            e1.setClock(envclock)
            print(PID+'New  Environment Time = ',envclock)
           
            energy = a1.getEnergy()
            if energy <0:
               print(PID+'ATTENTION: Actor has no more Energy!!!')
               FLAGCYC = cyclesNo+1
           
            else: 
                TIMELEFT = cyclesNo - FLAGCYC                
                FLAGCYC = FLAGCYC + 1
                
               
            ## Update control window for parameter
            ## Deleting old values
               
            ctrl.ctrlWinDel(txtEnergy)            
            ctrl.ctrlWinDel(txtTimeLeft)     
            
            ## Writing new values
            
            txtEnergy,txtTimeLeft = ctrl.ctrlWinRunUpdate(winC2,energy,TIMELEFT)
               
            if debugYes == 1:             
                win.getMouse()
            else:
                tm.sleep(SLEEP)

###########################################################################
## END OF CYCLES
## KEEP VALUES FOR STATISTICS
## SHOW INTERMEDIARY VALUES
###########################################################################    
        
        ENERGYAR[FLAGREP-1] = int(energy/(EMAX/100))
        print(PID+'ENERGYAR in loop = ',ENERGYAR)
           

###########################################################################
# Increment for  loop Repetitions
###########################################################################
        FLAGCYC =1
        envclock= 1
        e1.setClock(envclock)
        FLAGREP = FLAGREP + 1
       
        print(PID+'FLAGREP = ',FLAGREP)
       
        # Delete actor from last experiment
        p=a1.getPos()
        env.deleteActor2(a1,win,distance,gr2)
       
    print(PID+'ENERGYAR = ',ENERGYAR)
  
    env.my_plotterMean(ENERGYAR, EMAX,repeatNo,OBST,FOOD,behaviorNo)
       

    win.getMouse()
    #close windows
    winC2.close()
    win.close()

main()
